@@ -31,22 +31,25 @@ from . import scoping
 
 class Binding(object):
 
-    def __init__(self, binding_key, proviser_fn, scope_id, binding_location):
+    def __init__(self, binding_key, proviser_fn, get_binding_target_desc_fn,
+                 scope_id, get_binding_loc_fn):
         self.binding_key = binding_key
         self.proviser_fn = proviser_fn
+        self.get_binding_target_desc_fn = get_binding_target_desc_fn
         self.scope_id = scope_id
-        self._binding_location = binding_location
+        self._get_binding_loc_fn = get_binding_loc_fn
 
     def __str__(self):
         return 'the binding at {0}, from {1} to {2}, in "{3}" scope'.format(
-            self._binding_location, self.binding_key,
-            self.proviser_fn._pinject_desc, self.scope_id)
+            self._get_binding_loc_fn(), self.binding_key,
+            self.get_binding_target_desc_fn(), self.scope_id)
 
 
 def _handle_explicit_binding_collision(
         colliding_binding, binding_key_to_binding, *pargs):
     other_binding = binding_key_to_binding[colliding_binding.binding_key]
-    raise errors.ConflictingExplicitBindingsError([colliding_binding, other_binding])
+    raise errors.ConflictingExplicitBindingsError(
+        [colliding_binding, other_binding])
 
 
 def _handle_implicit_binding_collision(
@@ -90,7 +93,8 @@ def get_overall_binding_key_to_binding_maps(bindings_lists):
             True: _handle_explicit_binding_collision,
             False: _handle_implicit_binding_collision}[is_final_index]
         this_binding_key_to_binding, this_collided_binding_key_to_bindings = (
-            _get_binding_key_to_binding_maps(bindings, handle_binding_collision_fn))
+            _get_binding_key_to_binding_maps(
+                bindings, handle_binding_collision_fn))
         for good_binding_key in this_binding_key_to_binding:
             collided_binding_key_to_bindings.pop(good_binding_key, None)
         binding_key_to_binding.update(this_binding_key_to_binding)
@@ -108,6 +112,19 @@ class BindingMapping(object):
         self._collided_binding_key_to_bindings = (
             collided_binding_key_to_bindings)
 
+    def verify_requirements(self, required_bindings):
+        for required_binding in required_bindings:
+            required_binding_key = required_binding.binding_key
+            if required_binding_key not in self._binding_key_to_binding:
+                if (required_binding_key in
+                    self._collided_binding_key_to_bindings):
+                    raise errors.ConflictingRequiredBindingError(
+                        required_binding,
+                        self._collided_binding_key_to_bindings[
+                            required_binding_key])
+                else:
+                    raise errors.MissingRequiredBindingError(required_binding)
+
     def get(self, binding_key, injection_site_desc):
         if binding_key in self._binding_key_to_binding:
             return self._binding_key_to_binding[binding_key]
@@ -153,12 +170,9 @@ def get_explicit_class_bindings(
     for cls in classes:
         if decorators.is_explicitly_injectable(cls):
             for arg_name in get_arg_names_from_class_name(cls.__name__):
-                binding_key = binding_keys.new(arg_name)
-                proviser_fn = create_class_proviser_fn(
-                    locations.get_type_loc(cls), binding_key, cls)
-                explicit_bindings.append(Binding(
-                    binding_key, proviser_fn, scoping.DEFAULT_SCOPE,
-                    locations.get_type_loc(cls)))
+                explicit_bindings.append(new_binding_to_class(
+                    binding_keys.new(arg_name), cls, scoping.DEFAULT_SCOPE,
+                    lambda cls=cls: locations.get_loc(cls)))
     return explicit_bindings
 
 
@@ -167,14 +181,15 @@ def get_provider_bindings(
         get_arg_names_from_provider_fn_name=(
             providing.default_get_arg_names_from_provider_fn_name)):
     provider_bindings = []
-    fns = inspect.getmembers(binding_spec, lambda x: type(x) == types.MethodType)
+    fns = inspect.getmembers(binding_spec,
+                             lambda x: type(x) == types.MethodType)
     for _, fn in fns:
         default_arg_names = get_arg_names_from_provider_fn_name(fn.__name__)
         fn_bindings = get_provider_fn_bindings(fn, default_arg_names)
         for binding in fn_bindings:
             if binding.scope_id not in known_scope_ids:
                 raise errors.UnknownScopeError(
-                    binding.scope_id, locations.get_class_name_and_loc(fn))
+                    binding.scope_id, locations.get_name_and_loc(fn))
         provider_bindings.extend(fn_bindings)
     return provider_bindings
 
@@ -187,12 +202,9 @@ def get_implicit_class_bindings(
     for cls in classes:
         arg_names = get_arg_names_from_class_name(cls.__name__)
         for arg_name in arg_names:
-            binding_key = binding_keys.new(arg_name)
-            proviser_fn = create_class_proviser_fn(
-                locations.get_type_loc(cls), binding_key, cls)
-            implicit_bindings.append(Binding(
-                binding_key, proviser_fn, scoping.DEFAULT_SCOPE,
-                locations.get_type_loc(cls)))
+            implicit_bindings.append(new_binding_to_class(
+                binding_keys.new(arg_name), cls, scoping.DEFAULT_SCOPE,
+                lambda cls=cls: locations.get_loc(cls)))
     return implicit_bindings
 
 
@@ -207,10 +219,12 @@ class Binder(object):
     def bind(self, arg_name, annotated_with=None,
              to_class=None, to_instance=None, in_scope=scoping.DEFAULT_SCOPE):
         if in_scope not in self._scope_ids:
-            raise errors.UnknownScopeError(in_scope, locations.get_back_frame_loc())
+            raise errors.UnknownScopeError(
+                in_scope, locations.get_back_frame_loc())
         binding_key = binding_keys.new(arg_name, annotated_with)
-        specified_to_params = ['to_class' if to_class is not None else None,
-                               'to_instance' if to_instance is not None else None]
+        specified_to_params = [
+            'to_class' if to_class is not None else None,
+            'to_instance' if to_instance is not None else None]
         specified_to_params = [x for x in specified_to_params if x is not None]
         if not specified_to_params:
             binding_loc = locations.get_back_frame_loc()
@@ -223,42 +237,52 @@ class Binder(object):
         # TODO(kurts): this is such a hack; isn't there a better way?
         if to_class is not None:
             @decorators.annotate_arg('_pinject_class', (to_class, in_scope))
-            @decorators.provides(annotated_with=annotated_with, in_scope=in_scope)
+            @decorators.provides(annotated_with=annotated_with,
+                                 in_scope=in_scope)
             def provide_it(_pinject_class):
                 return _pinject_class
             with self._lock:
                 self._collected_bindings.extend(
                     get_provider_fn_bindings(provide_it, [arg_name]))
                 if (to_class, in_scope) not in self._class_bindings_created:
-                    self._collected_bindings.append(Binding(
-                        binding_keys.new('_pinject_class', (to_class, in_scope)),
-                        create_class_proviser_fn(locations.get_back_frame_loc(),
-                                                 binding_key, to_class),
-                        in_scope, locations.get_type_loc(to_class)))
+                    back_frame_loc = locations.get_back_frame_loc()
+                    self._collected_bindings.append(new_binding_to_class(
+                        binding_keys.new('_pinject_class',
+                                         (to_class, in_scope)),
+                        to_class, in_scope, lambda: back_frame_loc))
                     self._class_bindings_created.append((to_class, in_scope))
         else:
-            proviser_fn = create_instance_proviser_fn(binding_key, to_instance)
             back_frame_loc = locations.get_back_frame_loc()
             with self._lock:
-                self._collected_bindings.append(Binding(
-                    binding_key, proviser_fn, in_scope, back_frame_loc))
+                self._collected_bindings.append(new_binding_to_instance(
+                    binding_key, to_instance, in_scope,
+                    lambda: back_frame_loc))
 
 
-def create_class_proviser_fn(binding_loc, binding_key, to_class):
+def new_binding_to_class(binding_key, to_class, in_scope, get_binding_loc_fn):
     if not inspect.isclass(to_class):
         raise errors.InvalidBindingTargetError(
-            binding_loc, binding_key, to_class, 'class')
-    proviser_fn = lambda injection_context, obj_provider: obj_provider.provide_class(
-        to_class, injection_context)
-    class_name_and_loc = locations.get_class_name_and_loc(to_class)
-    proviser_fn._pinject_desc = 'the class {0}'.format(class_name_and_loc)
-    return proviser_fn
+            get_binding_loc_fn(), binding_key, to_class, 'class')
+    def Proviser(injection_context, obj_provider, pargs, kwargs):
+        return obj_provider.provide_class(
+            to_class, injection_context, pargs, kwargs)
+    def GetBindingTargetDesc():
+        return 'the class {0}'.format(locations.get_name_and_loc(to_class))
+    return Binding(binding_key, Proviser, GetBindingTargetDesc, in_scope,
+                   get_binding_loc_fn)
 
 
-def create_instance_proviser_fn(binding_key, to_instance):
-    proviser_fn = lambda injection_context, obj_provider: to_instance
-    proviser_fn._pinject_desc = 'the instance {0!r}'.format(to_instance)
-    return proviser_fn
+def new_binding_to_instance(
+        binding_key, to_instance, in_scope, get_binding_loc_fn):
+    def Proviser(injection_context, obj_provider, pargs, kwargs):
+        if pargs or kwargs:
+            raise TypeError('instance provider takes no arguments'
+                            ' ({0} given)'.format(len(pargs) + len(kwargs)))
+        return to_instance
+    def GetBindingTargetDesc():
+        return 'the instance {0!r}'.format(to_instance)
+    return Binding(binding_key, Proviser, GetBindingTargetDesc, in_scope,
+                   get_binding_loc_fn)
 
 
 class BindingSpec(object):
@@ -269,16 +293,26 @@ class BindingSpec(object):
     def dependencies(self):
         return []
 
+    def __eq__(self, other):
+        return type(self) == type(other)
+
+    def __hash__(self):
+        return hash(type(self))
+
 
 def get_provider_fn_bindings(provider_fn, default_arg_names):
     provider_decorations = decorators.get_provider_fn_decorations(
         provider_fn, default_arg_names)
-    proviser_fn = lambda injection_context, obj_provider: (
-        obj_provider.call_with_injection(provider_fn, injection_context))
-    proviser_fn._pinject_desc = 'the provider {0!r}'.format(provider_fn)
+    def Proviser(injection_context, obj_provider, pargs, kwargs):
+        return obj_provider.call_with_injection(
+            provider_fn, injection_context, pargs, kwargs)
+    def GetBindingTargetDescFn():
+        return 'the provider method {0}'.format(
+            locations.get_name_and_loc(provider_fn))
     return [
         Binding(binding_keys.new(provider_decoration.arg_name,
                                  provider_decoration.annotated_with),
-                proviser_fn, provider_decoration.in_scope_id,
-                locations.get_type_loc(provider_fn))
+                Proviser, GetBindingTargetDescFn,
+                provider_decoration.in_scope_id,
+                lambda p_fn=provider_fn: locations.get_loc(p_fn))
         for provider_decoration in provider_decorations]
