@@ -28,12 +28,15 @@ from . import injection_contexts
 from . import locations
 from . import object_providers
 from . import providing
+from . import required_bindings as required_bindings_lib
 from . import scoping
 
 
 def new_object_graph(
         modules=finding.ALL_IMPORTED_MODULES, classes=None, binding_specs=None,
         only_use_explicit_bindings=False, allow_injecting_none=False,
+        configure_method_name='configure',
+        dependencies_method_name='dependencies',
         get_arg_names_from_class_name=(
             bindings.default_get_arg_names_from_class_name),
         get_arg_names_from_provider_fn_name=(
@@ -53,6 +56,8 @@ def new_object_graph(
       only_use_explicit_bindings: whether to use only explicit bindings (i.e.,
           created by binding specs or @pinject.injectable, etc.)
       allow_injecting_none: whether to allow a provider method to provide None
+      configure_method_name: the name of binding specs' configure method
+      dependencies_method_name: the name of binding specs' dependencies method
       get_arg_names_from_class_name: a function mapping a class name to a
           sequence of the arg names to which those classes should be
           implicitly bound (if any)
@@ -106,6 +111,7 @@ def new_object_graph(
         explicit_bindings = bindings.get_explicit_class_bindings(
             found_classes, get_arg_names_from_class_name)
         binder = bindings.Binder(explicit_bindings, known_scope_ids)
+        required_bindings = required_bindings_lib.RequiredBindings()
         if binding_specs is not None:
             binding_specs = list(binding_specs)
             processed_binding_specs = set()
@@ -114,25 +120,38 @@ def new_object_graph(
                 if binding_spec in processed_binding_specs:
                     continue
                 processed_binding_specs.add(binding_spec)
+                all_kwargs = {'bind': binder.bind,
+                              'require': required_bindings.require}
+                configure_method = getattr(binding_spec, configure_method_name)
+                configure_kwargs = _pare_to_present_args(
+                    all_kwargs, configure_method)
+                if not configure_kwargs:
+                    raise errors.ConfigureMethodMissingArgsError(
+                        configure_method, all_kwargs.keys())
                 try:
-                    binding_spec.configure(binder.bind)
+                    configure_method(**configure_kwargs)
                     has_configure = True
                 except NotImplementedError:
                     has_configure = False
-                dependencies = binding_spec.dependencies()
-                binding_specs.extend(dependencies)
+                if hasattr(binding_spec, dependencies_method_name):
+                    dependencies_method = (
+                        getattr(binding_spec, dependencies_method_name))
+                    dependencies = dependencies_method()
+                    binding_specs.extend(dependencies)
                 provider_bindings = bindings.get_provider_bindings(
                     binding_spec, known_scope_ids,
                     get_arg_names_from_provider_fn_name)
                 explicit_bindings.extend(provider_bindings)
-                if not has_configure and not dependencies and not provider_bindings:
+                if (not has_configure and
+                    not dependencies and
+                    not provider_bindings):
                     raise errors.EmptyBindingSpecError(binding_spec)
-
         binding_key_to_binding, collided_binding_key_to_bindings = (
             bindings.get_overall_binding_key_to_binding_maps(
                 [implicit_class_bindings, explicit_bindings]))
         binding_mapping = bindings.BindingMapping(
             binding_key_to_binding, collided_binding_key_to_bindings)
+        binding_mapping.verify_requirements(required_bindings.get())
     except errors.Error as e:
         if use_short_stack_traces:
             raise e
@@ -185,6 +204,11 @@ def _verify_callable(fn, arg_name):
         raise errors.WrongArgTypeError(arg_name, 'callable', type(fn).__name__)
 
 
+def _pare_to_present_args(kwargs, fn):
+    arg_names, _, _, _ = inspect.getargspec(fn)
+    return {arg: value for arg, value in kwargs.iteritems() if arg in arg_names}
+
+
 class ObjectGraph(object):
     """A graph of objects instantiable with dependency injection."""
 
@@ -211,34 +235,10 @@ class ObjectGraph(object):
             raise errors.NonExplicitlyBoundClassError(provide_loc, cls)
         try:
             return self._obj_provider.provide_class(
-                cls, self._injection_context_factory.new(cls.__init__))
+                cls, self._injection_context_factory.new(cls.__init__),
+                direct_init_pargs=[], direct_init_kwargs={})
         except errors.Error as e:
             if self._use_short_stack_traces:
                 raise e
             else:
                 raise
-
-    # TODO(kurts): what's the use case for this, really?  Provider functions
-    # are already injected by default.  Functional programming?
-    def wrap(self, fn):
-        # This has to return a function with a different signature (and can't
-        # use @decorator) since otherwise python would require the caller to
-        # pass in all positional args that have no defaults, instead of
-        # letting those be injected if they're not passed in.
-        arg_names, unused_varargs, unused_keywords, defaults = inspect.getargspec(fn)
-        if defaults is None:
-            defaults = []
-        injectable_arg_names = arg_names[:(len(arg_names) - len(defaults))]
-        @functools.wraps(fn)
-        def WrappedFn(*pargs, **kwargs):
-            injected_arg_names = [
-                arg_name for index, arg_name in enumerate(injectable_arg_names)
-                if index >= len(pargs) and arg_name not in kwargs]
-            if injected_arg_names:
-                kwargs = dict(kwargs)
-                for arg_name in injected_arg_names:
-                    kwargs[arg_name] = self._obj_provider.provide_from_arg_binding_key(
-                        fn,  arg_binding_keys.new(arg_name),
-                        self._injection_context_factory.new(fn))
-            return fn(*pargs, **kwargs)
-        return WrappedFn
